#!/usr/local/bin/perl
# nwa2s2p.pl
# Converts data that are taken from the 8753D network analyzer and converts
# them into a two port s-parameter file (.s2p)
# note: I don't have any real reference on these file formats, I'm just
# interpreting it as I go (hence the form)
# 
# Copyright(C) 1997 Lance Lascari WS2B yadda yadda yadda
# ws2b@microwavedata.com
# ws2b@frontiernet.net
# http://www.frontiernet.net/~ws2b
# LL 6/6/97 - 6/7/97
# revised : 1/27/98 LL - added extra freq field width for narrow spans at low 
#                        freqs for things like IF filters
#6/8/97 LL - added checks to file name length
# bummer that perl doesn't support multi-dimensional arrays directly!
# Note 1: need to modify this so it works well on both win32 and UNIX platforms
# since the CR/LF combo is two chars
# Note 2: the precision that is written out may seem rediculous, but the first 
# thing I set out to measure was a very lossy SAW filter. With too few digits
# the simulator plots look horrible because of the truncation to zero 

# Command line:  nwa2s2p  inputfile outputname
#

# some configuration type stuff
$GHz=1e9;         # Hz divided by this gives us GHz for s2p file


# first verify command line and file openings
die("USAGE: nwa2s2p [nwafile.ext} [outputbasename]\r\n") 
    if( ($ARGV[0] eq "")  || ($ARGV[1] eq "")) ;

unless( open(INDATA,$ARGV[0])) 
{   
    die("can't open input data file $ARGV[0]\r\n");
}

#check output file name length
$len=length($ARGV[1]);
if( $len > 8 )
{
    die("ERROR: I'll only allow you 8 characters in the output file name\n");
}
unless( open(OUTFILE,">".$ARGV[1].".s2p"))     # append ".s2p" extension
{
     die("can't open output file $ARGV[1]\r\n");
}

printf("Enter one line of text to describe the data [enter when done]\n:");
$desc=<STDIN>;        # get this from the user
chop($desc);          # hack the newline off the text line

# print time in the header file as well as source filename
$tim=time();
$tim=localtime($tim);
if($desc eq "")           # check for null string
{
      $desc="I'm too lazy to enter a description!";
}
printf OUTFILE ("!!!!\n!INFO:%s\n!!!!",$desc);

printf OUTFILE ("! %s -> %s\n", $ARGV[0], $ARGV[1].".s2p");
printf OUTFILE ("! S-parameter file converted from network analyzer output by\n");
printf OUTFILE ("! Lance Lascari's Perl program $tim\n");
printf OUTFILE ("!!!!!!!!!!!!!!NWA data header follows\n");

# now get on to some useful stuff
printf(" NWA -> S2P converter by WS2B\n");
printf(" Input File: %s Output File :%s\n",$ARGV[0],  $ARGV[1].".s2p");
printf("no error checking yet, here's some of the header info\n");
for($i=0;$i<=7;$i++)
{
    $linebuf=<INDATA>;                                # get one line of input
    chop($linebuf);                                   #remove line feed
    printf(" line %02d reads :%s\n",$i,$linebuf);
    printf OUTFILE ("!%s\n",$linebuf);
}
printf OUTFILE ("!!!!!!!!!!!!!!\n");
printf OUTFILE ("# ghz s ma r 50\n");

# now get the frequency information and see if this stuff makes sense

$linebuf=<INDATA>;              # get one line of input
chop($linebuf);                 #remove line feed

# this is the statement before the frequency block
die("SEG_LIST_BEGIN match failed\n") 
    if ($linebuf ne "SEG_LIST_BEGIN"); 

# get the line that has SEG startfreq endfreq npts
$linebuf=<INDATA>;
chop($linebuf);                 #remove line feed
# now split it up into an array of strings (tokens separated by whitespace)
@strbuf=split(/ /,$linebuf);
# now check to see that this line begins with the "SEG" statement
die("SEG match failed\n") 
    if (@strbuf[0] ne "SEG"); 

die("[freq] start, stop or numpoints was zero\n") 
    if (@strbuf[1] eq 0 ||@strbuf[2] eq 0 ||@strbuf[3] eq 0 );

# now put the frequencies together in real scalar variables and calculate
# the frequency step
$startfreq=(@strbuf[1]/$GHz);
$stopfreq=(@strbuf[2]/$GHz);
$npts=@strbuf[3];
$freqstep=(($stopfreq-$startfreq)/($npts -1));

printf("start=%08.4f end=%08.4f npts=%d fstep=%08.4f\n",$startfreq,$stopfreq,$npts,$freqstep);

#
# this is the statement after the frequency block
$linebuf=<INDATA>;
chop($linebuf);                 #remove line feed
die("SEG_LIST_END match failed\n") 
    if ($linebuf ne "SEG_LIST_END"); 


# first create the master output array and fill the frequency slot
# fill the rest with zero
for($i=0;$i<$npts;$i++)
{
    @freq[$i]=($startfreq+($i*$freqstep));
    @MS11[$i]=0;
    @AS11[$i]=0;
    @MS21[$i]=0;
    @AS21[$i]=0;
    @MS12[$i]=0;
    @AS12[$i]=0;
    @MS22[$i]=0;
    @AS22[$i]=0;
}

###### first data will be S11
# test for the beginning of the data block
&testbeg();
for($i=0;$i<$npts;$i++)
{
    @temp=&getpair();
    @MS11[$i]=@temp[0];
    @AS11[$i]=@temp[1];
}
&testend();

#next s21
&testbeg();
for($i=0;$i<$npts;$i++)
{
    @temp=&getpair();
    @MS21[$i]=@temp[0];
    @AS21[$i]=@temp[1];
}
&testend();

#now S12
&testbeg();
for($i=0;$i<$npts;$i++)
{
    @temp=&getpair();
    @MS12[$i]=@temp[0];
    @AS12[$i]=@temp[1];
}
&testend();

#finally S22
&testbeg();
for($i=0;$i<$npts;$i++)
{
    @temp=&getpair();
    @MS22[$i]=@temp[0];
    @AS22[$i]=@temp[1];
}
&testend();


# for now a crude routine to write the s-params out

for($i=0;$i<$npts;$i++)
{
#freq MS11 AS11 MS21 AS21 MS12 AS12 MS22 AS22
 printf OUTFILE ("%010.08f  %08.06f  %08.06f  %08.06f  %08.06f  %08.06f  %08.06f  %08.06f  %08.06f\n",@freq[$i],@MS11[$i],@AS11[$i],@MS21[$i],@AS21[$i],@MS12[$i],@AS12[$i],@MS22[$i],@AS22[$i]);

}

close(INDATA);
close(OUTFILE);
printf("\nThe End\n");



############# subroutines

#######
#getpair()   
# -reads one pair of RE IM data from input file, converts to polar & returns
#######
sub getpair
{
    my($Re,$Im);       # get these from file
    my($mag,$ang);        # local vars to be returned
    $buf=<INDATA>;   #input line is read from file
    chop($buf);         # chop it here.
    @pair=split(/,/,$buf);      # parse the real and imag out
    $Re=@pair[0]/1.0;
    $Im=@pair[1]/1.0;

    $mag=sqrt(($Re * $Re) + ($Im * $Im));     # magnitude calculated
    $ang=57.3 * atan2( $Im,$Re);              # angle calculated -> degrees
#    printf("%6.3f,%6.3f\n",$mag,$ang);
    return($mag,$ang)
}
#######


#######
#testbeg()   
# -gets a line and sees if it reads "BEGIN" (to verify data segments)
# dies if it isn't
#######
sub testbeg
{
    $buf=<INDATA>;       # get the line from a file
    chop($buf);         # chop it here.
    die("no beginning found bozo!\n") if ($buf ne "BEGIN");
}
#######

#######
#testend()   
# -gets a line and sees if it reads "END" (to verify data segments)
# dies if it isn't
#######
sub testend
{
    $buf=<INDATA>;       # get the line from a file
    chop($buf);         # chop it here.
    die("no end found bozo!($buf)\n") if ($buf ne "END");
}
